import java.net.*;
import java.io.*;
import java.util.*;
import postgres.*;
//import PGconnection;


class PGtest {
	static public void main(String argv[]) {
		PGresult res;
		PGconnection pgConn = new PGconnection();
		Date today = new Date();

		today = new Date("Sat Mar 23 09:00:00  1996");
		System.out.println("This is Java-Postgres95 version "+PGconnection.version());

		res = pgConn.PQsetdb("localhost",5432,"","","temp","kellyj");
		if((res.getErrorMessage()).length()>0) {
			System.err.println(res.getErrorMessage());
			System.exit(1);
		}

		/* Clean up from any previous crash */
		res=pgConn.PQexec("drop table TEMP");
		res.printTuples();

		/* Make a TEMP table */
		res=pgConn.PQexec("create table TEMP ( a int, b varchar(10), c abstime)");
		res.printTuples();
		res=pgConn.PQexec("select * from TEMP");
		res.printTuples();

		/* Test a out of bounds request */
		try {
			System.out.println(res.getField(1,"a"));
		} catch (PGexception e) {
			System.out.println("Successfully caught PGexception "+e.toString());
		}

		/* Test an insertion */
		res=pgConn.PQexec("insert into TEMP (a,  b, c) values (1, 'Hello', '"+today.toString()+"')");
		res.printTuples();
		res=pgConn.PQexec("select * from TEMP");
		res.printTuples();

		/* Test a bad field name */
		try {
			System.out.println(res.getField(0,"junk"));
		} catch (PGexception e) {
			System.out.println("Successfully caught PGexception "+e.toString());
		}


		/* Test COPY IN */
		res=pgConn.PQexec("copy TEMP from stdin");
		res.printTuples();
		pgConn.PQputline("2\tGeorge\tJan 1 1996\n3\tWashington\tJan 1 1996\n4\tBill\tJan 1 1996\n5\tClinton\tJan 1 1996\n.\n");
		pgConn.PQendcopy();

		/* Test COPY OUT */
		System.out.println("\nTest of Copy out");
		res=pgConn.PQexec("copy TEMP to stdout");
		String sOutput;
		sOutput = pgConn.PQgetline();
		while(!sOutput.equals(".")) {
			System.out.println(sOutput);
			sOutput = pgConn.PQgetline();
		}
		pgConn.PQendcopy();
		System.out.println("End Test of Copy out");
			
		/* Test PORTALS */
		System.out.println("\nTest of Portals");
		res = pgConn.PQexec("begin transaction");
		res.printTuples();
		res = pgConn.PQexec("declare myportal cursor for select * from TEMP");
		res.printTuples();
		res = pgConn.PQexec("fetch forward 2 in myportal");
		res.printTuples();
		res = pgConn.PQexec("fetch forward all in myportal");
		res.printTuples();
		res = pgConn.PQexec("end transaction");
		res.printTuples();
		System.out.println("\nEnd Test of Portals");

		/* Test CREATE INDEX */
		System.out.println("\nTest of CREATE INDEX");
		res = pgConn.PQexec("create index TEMP_A on TEMP using btree ( a int4_ops )");
		res.printTuples();
		res = pgConn.PQexec("drop index TEMP_A"); 
		res.printTuples();
		System.out.println("\nEnd Test of CREATE INDEX");


		/* Test PGrecord class */
		System.out.println("\nTest of PGrecord");
		res=pgConn.PQexec("select * from TEMP where a=2");
		res.printTuples();
		Vector v = res.getPGvector();
		PGrecord d;
		String sInsert=new String();
		String sUpdate=new String();
		for(Enumeration e=v.elements(); e.hasMoreElements();) {
			d = (PGrecord)(e.nextElement());
			sUpdate = d.updateSQL("TEMP","a");
			sInsert = d.insertSQL("TEMP");
			System.out.println(sInsert);
			System.out.println(sUpdate);
			for(int i=0;i<d.field.length;i++) {
				System.out.println(d.field[i].name+"="+d.data[i]);
			}
			Object a, b, c;
			a = d.getValue("a");
			b = d.getValue("b");
			c = d.getValue("c");
			System.out.println(a.getClass().getName()+" is a's class, value="+a);
			System.out.println(b.getClass().getName()+" is b's class, value="+b);
			System.out.println(c.getClass().getName()+" is c's class, value="+c);
			d.setValue("b", "Ghandi");
			d.setValue("c", "Jul 1 1948");
			sUpdate = d.updateSQL("TEMP","a");
			System.out.println(sUpdate);
			d.setValue("a", new Integer(6));
			sInsert = d.insertSQL("TEMP");
			System.out.println(sInsert);
		}

		res=pgConn.PQexec(sInsert);
		res=pgConn.PQexec(sUpdate);
		res.printTuples();
		res=pgConn.PQexec("select * from TEMP");
		res.printTuples();

		res=pgConn.PQexec("delete from TEMP ");
		res.printTuples();
		res=pgConn.PQexec("select * from TEMP");
		res.printTuples();

		res=pgConn.PQexec("drop table TEMP");
		res.printTuples();

		pgConn.PQfinish();
	}
}

/*
 * This is an attempt to support large objects, which don't yet work
 *

class LOtest {
	public static void main(String argv[]) {
		
		PGconnection pgConn = new PGconnection();
		PGresult res;

		System.out.println("This is Java-Postgres95 version "+PGconnection.version());

		res = pgConn.PQsetdb("localhost",5432,"","","temp","kellyj");
		if((res.getErrorMessage()).length()>0) {
			System.err.println(res.getErrorMessage());
			System.exit(1);
		}

		res=pgConn.PQexec("create table IMAGE ( name text, raster oid)");
		res.printTuples();
		res=pgConn.PQexec("insert into IMAGE (name,raster) values ('pwd',lo_import('/etc/passwd'))");
		res.printTuples();
		res=pgConn.PQexec("select * from IMAGE where name='pwd' ");
		res.printTuples();
		
		Integer iOid  = new Integer (res.getField(0,"raster"));
		System.out.println("Integer Value: "+iOid);
		int fd;

		fd = pgConn.lo_open(iOid.intValue(),0x00040000);
		System.out.println("FD returned: "+fd);
		System.out.println("LO_READ: <"+pgConn.lo_read(fd,1000)+">");
		pgConn.lo_close(fd);

		res=pgConn.PQexec("drop table IMAGE ");
		res.printTuples();

	}
}
 */

