/*
    PGconnection.java - A Java class to manage Postgres95 connection
    Copyright (C) 1996  John F. Kelly

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package postgres;

import java.net.*;
import java.io.*;

/**
 * PGconnection opens connections to the database and parses the
 * output stream to create PGresult objects (@see PGresult.html)
 *
 * @author John F. Kellyj
 * @version 0.2
 */

public
class PGconnection {
	static String _version = "0.2";
	public static String version() { return _version; }
	private Socket sockPG;
	private InputStream isPG;
	private OutputStream osPG;
	int iDebug;
	int iBytesRead;
	byte bResult[];

	public PGconnection() {}

	/** 
	 * pqGetInt is a private method that gets an integer
	 * of a specifiable size from the input stream
         * @param b - the number of bytes to retrieve
	 * @return the integer retrieved
         */
	private int pqGetInt(int b) {
		int j, ret, factor;
		ret =0;
		factor = 1;
		try {
			for(j=0;j<b;j++) {
				ret += factor*isPG.read();
				factor *= 256;
			}
		} catch (Exception e ) {
			System.out.println(e.toString());
		}
		return ret;
	}

	/** 
	 * pqPutInt is a private method that gets an integer
	 * of a specifiable size from the input stream
         * @param iLength - the number of bytes to send
	* iValue - the value to send in those bytes
         */
	private void pqPutInt(int iLength, int iValue) {
		int j,  r;
		byte bSend[] = new byte[iLength];

		r = iValue;
		try {
			for(j=0;j<iLength;j++) {
				bSend[j] = (byte)(r % 256);
				r /= 256;	
			}
			osPG.write(bSend);
		} catch (Exception e ) {
			System.out.println(e.toString());
		}
	}

	/** 
	 * pqGets is a private method that gets a String  
	 * from the input stream.  THe input string is null terminated
	 * and can be no greater than 256 bytes.
         */
	private String pqGets() {
		byte buf[] = new byte[256];
		int i;
		String ret;

		i = 0;

		try {
			buf[0] = (byte)isPG.read(); 
			while( buf[i]!=(byte)0) {
				i++;
				buf[i] = (byte)isPG.read();
			}
			ret = new String(buf,0,0,i);
		} catch (Exception e) {
			ret = new String();
			System.out.println(e.toString());
		}
		return ret;
	}


	/** 
	 * pqPuts is a private method that puts a String  
	 * to the output stream (i.e. to the database).  
	 * @param sSend - the String to send
         */
	private void pqPuts(String sSend) {
		byte buf[] = new byte[sSend.length()+1];

		sSend.getBytes(0,sSend.length(),buf,0);
		buf[sSend.length()] = (byte)0;

		try {
			osPG.write(buf);
		} catch (Exception e) {
			System.out.println(e.toString());
		}
	}

	/** 
	 * pqfn sends a function identifier and an object array to the
 	 * the database. This is mostly used by the large object functions.
	 * @param iFnid - the integer identifier, oArgv[] - an array of 
	 * arguments (Integers or Strings), bInsInt - a boolean flag
	 * @result a PGresult
	 */
	private PGresult pqFn(int iFnid, Object oArgv[], boolean bIsInt) {
		PGresult res = new PGresult();
		int i;

		pqPuts("F ");
		pqPutInt(4,iFnid);
		pqPutInt(4,oArgv.length);

		for(i=0;i<oArgv.length;i++) {
			if(oArgv[i].getClass().getName().compareTo("java.lang.String")==0) {
				pqPutInt(4,((String)oArgv[i]).length());
				pqPuts((String)(oArgv[i]));
			} else if (oArgv[i].getClass().getName().compareTo("java.lang.Integer")==0) {
				pqPutInt(4,4);
				pqPutInt(4,((Integer)oArgv[i]).intValue());
			} else {
				System.err.println("pqFn: don't know how to send "+oArgv[i].getClass().getName());

			}		
		
		}
	
		try {
			byte bId;
			String sRetValue[] = new String[1];

			osPG.flush();

			bId = (byte) isPG.read();	
			if(bId != 'V') {
				if(bId=='E') {
					return new PGresult(pqGets());
				} else {
					System.err.println("pqFn: expected V, got:"+bId);
					System.exit(1);
					return new PGresult("pqFn: expected V, got"+bId);
				}
			}
			bId = (byte) isPG.read();	
			for(;;) {
				int c;
				switch(bId) {
				case 'G':
					int iActResLen = pqGetInt(4);
					System.out.println("VG Length: "+iActResLen);
					res.addField(0,new PGfield("Result",0,0));
					if(bIsInt) {
						Integer ii = new Integer(pqGetInt(4));
						sRetValue[0] = ii.toString();
						System.out.println("GOT: "+ii);
					} else {
						byte buf[] = new byte[iActResLen];
						isPG.read(buf);
						sRetValue[0] = new String(buf,0);
					}
					// Read off trailing 0x30
					isPG.read();
					res.addTuple(0,sRetValue);
					return res;
				default:
				case 'E':
					res.setErrorMessage("pqFn: Returned an Error");
					System.err.println("pqFn error-id: "+bId);
					System.exit(1);
				
					return res;
				case 'N':
					sRetValue[0] = pqGets();
					break;
				case (byte)-1:
					return res;
				}
			}
		} catch ( Exception e) {
			System.out.println(e.toString());
			System.exit(1);
			return res;
		}
	}
	
	int lo_seek(int fd, int offset, int whence ) {
		PGresult res;
		Object oArgv[] = new Object[3];
		
		oArgv[0] = new Integer(fd);
		oArgv[1] = new Integer(offset);
		oArgv[2] = new Integer(whence);
		res = pqFn(956,oArgv,true);
		return 0;
	}

	public int lo_open(int iLoOid, int iMode) {
		Object oArgv[] = new Object[2];
		PGresult res;
		String sRet[];
		int iRet, i;
		byte bRet[];	
		
		oArgv[0] = new Integer(iLoOid);
		oArgv[1] = new Integer(iMode);
		res = pqFn(952,oArgv,true);

		sRet = res.getTuple(0);
		iRet = (new Integer(sRet[0])).intValue();
		lo_seek(iRet, 0, 0);
		return iRet;
	}
	public String lo_read(int fd, int iNumber) {
		Object oArgv[] = new Object[2];
		PGresult res;
		byte bRet[] = new byte[iNumber];

		oArgv[0] = new Integer(fd);
		oArgv[1] = new Integer(iNumber);
		res = pqFn(954,oArgv,false);
		String sRet = (res.getTuple(0))[0];
		return sRet;
	}
		
	public int lo_write(int fd, String sSend) {
		Object oArgv[] = new Object[2];
		PGresult res;

		oArgv[0] = new Integer(fd);
		oArgv[1] = sSend;
		res = pqFn(955,oArgv,true);
		String sRet[] = (res.getTuple(0));
		int iRet = (new Integer(sRet[0])).intValue();
		return iRet;
	}
		
		
	public int lo_close(int fd) {
		Object oArgv[] = new Object[1];
		PGresult res;
		
		oArgv[0] = new Integer(fd);
		res = pqFn(953,oArgv,true);
		return 0;
	}

	/** 
	 * getTuple  is called from the pqMakeTuples routine
	 * It gets a record of returned data, and returns that 
	 * as a String[]. 
	 * @param numFields - the number of fields in the record,
	 * bBinary - boolean flag indicating format:
	 * true for Binary data, false for ASCII
	 */
	private String[] getTuple( int numFields, boolean bBinary) {
		int bitmasksize = (numFields+7)/8;
		byte bitmap[] = new  byte[bitmasksize];
		byte bmap;
		byte tmp[];
		int i;
		String ret[] = new String[numFields];

		
		try {
			isPG.read(bitmap);
			bmap = bitmap[0];
			for(i=0;i<numFields;i++) {
				if(i%8==0) { bmap=bitmap[(i/8)]; }
				if( (bmap &(byte) 0x80)== 0) {
					tmp = new byte[0];
				} else {
					int len = pqGetInt(4);
					if(!bBinary) {
						len -= 4;
					}
					tmp = new byte[len];
					// Attempts to read 0 bytes will block forever!
					if(len>0) {
						isPG.read(tmp);
						ret[i] = new String(tmp,0);
					} else {
						ret[i] = new String();
					}
				}
				bmap *=2; 
			}
		} catch ( Exception e) {
			System.out.println(e.toString());
		}
		
		return ret; 
	}
	/**
	 * pqMakeTuples - called from PQexec to gather query results
 	 * @param sPortalName - String name of the portal that holds
	 * the query results
	 * @return a PGresult
	 */
	private PGresult pqMakeTuples(String sPortalName) {
		int numAttributes;
		int i;
		String tup[];
		byte id;
		PGresult res = new PGresult();

		res.setPortalName(sPortalName);

		numAttributes = pqGetInt(2);

		for(i=0;i<numAttributes;i++) {
			int type, length;
			String name = pqGets();
			length = pqGetInt(4);
			type = pqGetInt(2);
			res.addField(i,new PGfield(name,length,type));
		}	

		i = 0;
		while(true) {
			try {
				id = (byte)isPG.read();
			} catch ( Exception e) {
				id = (byte) 'E';
				System.out.println(e.toString());
			}
			
			switch(id) {
			case 'B':
			case 'D':
				tup = getTuple(numAttributes, (id == (byte)'B'));
				res.addTuple(i, tup);
				i++;
				break;
			case 'C':
				res.setCommandName(pqGets());
				return res;
			}
		}
			
	}
		
	/** 
	 * PQfinish - closes the connection to the database server
	 */
	public int PQfinish() {
		try {
			byte x[] = new byte [2];
			x[0] = (byte)'X';
			x[1] = (byte) 0;
			try {
				osPG.write(x);
			} catch (Exception e) {
				System.err.println(e.toString());
			}
			isPG.close();
			osPG.close();
			sockPG.close();
		} catch( Exception e ) {
			System.out.print(e.toString());
		}
		return 0;
	}

	/** 
	 * PQgetline - gets a "Copy Out" line from the database
	 * @return the line gathered from the input stream.
	 */
	public String PQgetline() {
		byte buf[] = new byte[1024];
		int i=0;
		
		while(true) {
			try {
				buf[i] = (byte)isPG.read();	
			} catch (Exception e) {
				System.err.println(e.toString());
				buf[i] = (byte)-1;
			}
			switch (buf[i]) {
			case (byte)'\n': //End of Line
				return new String(buf,0,0,i);
			case (byte)-1:
				//Got EOF or Exception
				return new String();
			default:
				i++;
				break;
			}
			if(i==1024) { // End of Buffer
				return new String(buf,0,0,i);
			}
		}
	}
	/**
	 * PQputline - sends a "Copy In" line to the database
	 * @param -  sSend a tab delimited, newline terminated string to 
	 * send to the database
	 */
	public void PQputline(String sSend) {
		byte buf[] = new byte[sSend.length()];
		sSend.getBytes(0,sSend.length(), buf,0);
		try {
			osPG.write(buf);
			osPG.flush();
		} catch(Exception e) {
			System.err.println(e.toString());
		}
	}
	/**
	 * PQendcopy - ends copy in/out mode
	 */
	public int PQendcopy() {
		byte id[] = new byte[1];

		try {
			isPG.read(id);
		} catch(Exception e) {
			System.err.println(e.toString());
		}
		switch (id[0]) {
		case (byte)'Z': // Finished copy
			return 1;
		default:
			System.err.println("PQendcopy error, id="+new String(id,0));
			return 0;
		}
	}
		

	/** 
	 * PQexec - sends SQL text to the database, returns a PGresult
	 */
	public PGresult PQexec(String sQuery) {
		byte bQuery[] = new byte[sQuery.length()+2];
		PGresult res ;
		String sPortalName = new String("blank");
		boolean bMore = true;
		
		sQuery.getBytes(0,sQuery.length(),bQuery,1);
		bQuery[0]=(byte)'Q';
		bQuery[bQuery.length-1] = (byte)0;

		// Send & Recieve
		try {
			osPG.write(bQuery);
			while(bMore) {
				isPG.read(bResult,0,1);
				switch(bResult[0]) {
				case 'I':
					res = new PGresult();
					res.setCommandName("EmptyQuery");
					// Read off trailing 0
					isPG.read(bResult,0,1);
					if(bResult[0]!=(byte)0) {
						res.setErrorMessage("Unexpected character after I");
					}
					return res;
				
				case 'C':
					// Portal Query Command, No Tuples returned
					byte bTmp[] = new byte[3];
					boolean bWait = true;
					String sTmp;
					sTmp=pqGets();
					res = new PGresult();
					res.setCommandName(sTmp);


					bTmp[0] = (byte)'Q';
					bTmp[1] = (byte)' ';
					bTmp[2] = (byte)0;
					osPG.write(bTmp);
					while(bWait) {
						sTmp = pqGets();
						bWait = (sTmp.charAt(0) != 'I');
					}
//					res.setCommandName("CommandOK");
					return res;
				case 'E':
					return new PGresult(pqGets());
				case 'P':
					sPortalName = pqGets();
					break;
				case 'B':
					res = new PGresult();
					res.setCommandName("CopyOut");
					return res;
				case 'D':
					res = new PGresult();
					res.setCommandName("CopyIn");
					return res;
				case 'T':
					return pqMakeTuples(sPortalName);
				default:
					return new PGresult("Unknown character recieved");
				}
			}
		} catch( Exception e ) {
			System.out.print(e.toString());
		}

		return new PGresult();
	}
	
	/**
	 * PQsetdb - opens a connection to a Postgres95 server
	 * @param pghost - String name of the postgres server machine,
	 * pgport - int port on which the database server listens,
	 * pgoptions (unused), pgtty - TTY on which to display debugging,
	 * dbname - String name of database, user - String name of user
	 */
	public PGresult PQsetdb(String pghost, int pgport, String pgoptions, 
			    String pgtty, String dbname, String user) {
		byte bStartup[] = new byte [288];
		PGresult res;

		bStartup[2]=1; bStartup[3]=32;
		bStartup[7] = 7;
		dbname.getBytes(0,dbname.length(),bStartup, 8);
		user.getBytes(0,user.length(),bStartup, 64+8);
		bResult = new byte[1024];
		
		iDebug=0;
		if(iDebug!=0) {
			int i;
			for(i=0;i<25;i++) {
				System.out.println(i+" "+(int)bStartup[i]);
			}
		}
		
		try {
			int bRead;

			sockPG = new Socket(pghost,pgport);
			isPG = sockPG.getInputStream();
			osPG = sockPG.getOutputStream();

			// Send the startup 
			osPG.write(bStartup,0,288);

			// Send an empty query to test connectivity
			return PQexec(" ");


		} catch ( Exception e) {
			System.out.println(e.toString());
			PQfinish();
			return new PGresult(e.toString());
		}
	}
}

		
